/**
 * Copyright (C) 2016 Deepin Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 **/

#include "helloworldplugin.h"

#include <QWidget>
#include <QLabel>
#include <QResizeEvent>

#include "hellopopup.h"

static const QString MenuIdOpenEyes = "menu_id_open_eyes";
static const QString MenuIdCloseEyes = "menu_id_close_eyes";

HelloWorldPlugin::HelloWorldPlugin() :
    QObject(nullptr),
    m_icon(new QLabel),
    m_tooltip(new QLabel("Hello World")),
    m_applet(new HelloPopup)
{
    QPixmap pix(":/deepin.png");
    m_icon->setPixmap(pix);
    m_icon->setMargin(6);
    m_icon->setAlignment(Qt::AlignCenter);
    m_icon->setScaledContents(true);
    m_icon->installEventFilter(this);

    m_tooltip->setStyleSheet("color:white;"
                             "padding:5px 10px;");
}

const QString HelloWorldPlugin::pluginName() const
{
    return "helloworld";
}

void HelloWorldPlugin::init(PluginProxyInterface *proxyInter)
{
    m_proxyInter = proxyInter;

    proxyInter->itemAdded(this, "");
}

QWidget *HelloWorldPlugin::itemWidget(const QString &itemKey)
{
    Q_UNUSED(itemKey);
    return m_icon;
}

QWidget *HelloWorldPlugin::itemTipsWidget(const QString &itemKey)
{
    Q_UNUSED(itemKey);
    return m_tooltip;
}

QWidget *HelloWorldPlugin::itemPopupApplet(const QString &itemKey)
{
    Q_UNUSED(itemKey);
    return m_applet;
}

const QString HelloWorldPlugin::itemContextMenu(const QString &itemKey)
{
    Q_UNUSED(itemKey)

    QList<QVariant> items;
    items.reserve(1);

    QMap<QString, QVariant> open;
    open["itemId"] = MenuIdOpenEyes;
    open["itemText"] = "Open Eyes";
    open["isActive"] = true;
    items.push_back(open);

    QMap<QString, QVariant> close;
    close["itemId"] = MenuIdCloseEyes;
    close["itemText"] = "Close Eyes";
    close["isActive"] = true;
    items.push_back(close);

    QMap<QString, QVariant> menu;
    menu["items"] = items;
    menu["checkableMenu"] = false;
    menu["singleCheck"] = false;

    return QJsonDocument::fromVariant(menu).toJson();
}

void HelloWorldPlugin::invokedMenuItem(const QString &itemKey, const QString &menuId, const bool checked)
{
    Q_UNUSED(itemKey);
    Q_UNUSED(checked);
    if (menuId == MenuIdOpenEyes) {
        QProcess::startDetached("xeyes");
    } else if (menuId == MenuIdCloseEyes) {
        QProcess::startDetached("killall xeyes");
    }
}

bool HelloWorldPlugin::eventFilter(QObject *watched, QEvent *event)
{
    if (watched == m_icon && event->type() == QEvent::MouseButtonPress) {
        QMouseEvent *mouse = static_cast<QMouseEvent*>(event);
        if (mouse->button() == Qt::RightButton) {
            m_proxyInter->requestContextMenu(this, "");
            return true;
        }
    }

    return false;
}

#if QT_VERSION < 0x050000
Q_EXPORT_PLUGIN2(helloworld, HelloWorldPlugin)
#endif // QT_VERSION < 0x050000
